// Generated from libavfilter/opencl/overlay.cl
const char *ff_source_overlay_cl =
"/*\n"
" * This file is part of FFmpeg.\n"
" *\n"
" * FFmpeg is free software; you can redistribute it and/or\n"
" * modify it under the terms of the GNU Lesser General Public\n"
" * License as published by the Free Software Foundation; either\n"
" * version 2.1 of the License, or (at your option) any later version.\n"
" *\n"
" * FFmpeg is distributed in the hope that it will be useful,\n"
" * but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
" * Lesser General Public License for more details.\n"
" *\n"
" * You should have received a copy of the GNU Lesser General Public\n"
" * License along with FFmpeg; if not, write to the Free Software\n"
" * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA\n"
" */\n"
"\n"
"__constant sampler_t sampler = (CLK_NORMALIZED_COORDS_FALSE |\n"
"                                CLK_FILTER_NEAREST);\n"
"\n"
"__kernel void overlay_pass(__write_only image2d_t dst,\n"
"                           __read_only  image2d_t main)\n"
"{\n"
"    int2 loc = (int2)(get_global_id(0), get_global_id(1));\n"
"\n"
"    float4 val = read_imagef(main, sampler, loc);\n"
"    write_imagef(dst, loc, val);\n"
"}\n"
"\n"
"__kernel void overlay_noalpha(__write_only image2d_t dst,\n"
"                              __read_only  image2d_t main,\n"
"                              __read_only  image2d_t overlay,\n"
"                              int x_position,\n"
"                              int y_position)\n"
"{\n"
"    int2 overlay_size = get_image_dim(overlay);\n"
"    int2 loc = (int2)(get_global_id(0), get_global_id(1));\n"
"\n"
"    if (loc.x <  x_position ||\n"
"        loc.y <  y_position ||\n"
"        loc.x >= overlay_size.x + x_position ||\n"
"        loc.y >= overlay_size.y + y_position) {\n"
"        float4 val = read_imagef(main, sampler, loc);\n"
"        write_imagef(dst, loc, val);\n"
"    } else {\n"
"        int2 loc_overlay = (int2)(x_position, y_position);\n"
"        float4 val       = read_imagef(overlay, sampler, loc - loc_overlay);\n"
"        write_imagef(dst, loc, val);\n"
"    }\n"
"}\n"
"\n"
"__kernel void overlay_alpha(__write_only image2d_t dst,\n"
"                            __read_only  image2d_t main,\n"
"                            __read_only  image2d_t overlay,\n"
"                            __read_only  image2d_t alpha,\n"
"                            int x_position,\n"
"                            int y_position,\n"
"                            int alpha_adj_x,\n"
"                            int alpha_adj_y,\n"
"                            int overlay_full)\n"
"{\n"
"    int2 overlay_size = get_image_dim(overlay);\n"
"    int2 alpha_size = get_image_dim(alpha);\n"
"    int2 loc = (int2)(get_global_id(0), get_global_id(1));\n"
"\n"
"    if (loc.x <  x_position ||\n"
"        loc.y <  y_position ||\n"
"        loc.x >= overlay_size.x + x_position ||\n"
"        loc.y >= overlay_size.y + y_position) {\n"
"        float4 val = read_imagef(main, sampler, loc);\n"
"        write_imagef(dst, loc, val);\n"
"    } else {\n"
"        int2 loc_overlay  = (int2)(x_position, y_position);\n"
"        float4 in_main    = read_imagef(main,    sampler, loc);\n"
"        float4 in_overlay = read_imagef(overlay, sampler, loc - loc_overlay);\n"
"\n"
"        int2 loc_alpha    = (int2)(loc.x * alpha_adj_x, loc.y * alpha_adj_y) - loc_overlay;\n"
"        float4 in_alpha   = read_imagef(alpha,   sampler, loc_alpha);\n"
"\n"
"#ifdef NEED_UNPREMUL\n"
"        float offset = overlay_full ? 0 : 0.0627f;\n"
"        in_overlay.x = (alpha_size.x == overlay_size.x && alpha_size.y == overlay_size.y) &&\n"
"                       (in_alpha.x > 0.0f && in_alpha.x < 1.0f)\n"
"                       ? min(max(in_overlay.x - offset, 0.0f) * 1.0f / in_alpha.x + offset, 1.0f)\n"
"                       : in_overlay.x;\n"
"\n"
"        in_overlay.x = (alpha_size.x > overlay_size.x && alpha_size.y > overlay_size.y) &&\n"
"                       (in_alpha.x > 0.0f && in_alpha.x < 1.0f)\n"
"                       ? min((in_overlay.x - 0.5f) * 1.0f / in_alpha.x + 0.5f, 1.0f)\n"
"                       : in_overlay.x;\n"
"#endif\n"
"        float4 val = in_overlay * in_alpha.x + in_main * (1.0f - in_alpha.x);\n"
"        write_imagef(dst, loc, val);\n"
"    }\n"
"}\n"
"\n"
"__kernel void overlay_noalpha_uv(__write_only image2d_t dst,\n"
"                                 __read_only  image2d_t main,\n"
"                                 __read_only  image2d_t overlay_u,\n"
"                                 __read_only  image2d_t overlay_v,\n"
"                                 int x_position,\n"
"                                 int y_position)\n"
"{\n"
"    int2 overlay_size = get_image_dim(overlay_u);\n"
"    int2 loc = (int2)(get_global_id(0), get_global_id(1));\n"
"\n"
"    if (loc.x <  x_position ||\n"
"        loc.y <  y_position ||\n"
"        loc.x >= overlay_size.x + x_position ||\n"
"        loc.y >= overlay_size.y + y_position) {\n"
"        float4 val = read_imagef(main, sampler, loc);\n"
"        write_imagef(dst, loc, val);\n"
"    } else {\n"
"        int2 loc_overlay = (int2)(x_position, y_position);\n"
"        float4 val_u     = read_imagef(overlay_u, sampler, loc - loc_overlay);\n"
"        float4 val_v     = read_imagef(overlay_v, sampler, loc - loc_overlay);\n"
"        write_imagef(dst, loc, (float4)(val_u.x, val_v.x, 0.0f, 1.0f));\n"
"    }\n"
"}\n"
"\n"
"__kernel void overlay_alpha_uv(__write_only image2d_t dst,\n"
"                               __read_only  image2d_t main,\n"
"                               __read_only  image2d_t overlay_u,\n"
"                               __read_only  image2d_t overlay_v,\n"
"                               __read_only  image2d_t alpha,\n"
"                               int x_position,\n"
"                               int y_position,\n"
"                               int alpha_adj_x,\n"
"                               int alpha_adj_y,\n"
"                               int overlay_full)\n"
"{\n"
"    int2 overlay_size = get_image_dim(overlay_u);\n"
"    int2 loc = (int2)(get_global_id(0), get_global_id(1));\n"
"\n"
"    if (loc.x <  x_position ||\n"
"        loc.y <  y_position ||\n"
"        loc.x >= overlay_size.x + x_position ||\n"
"        loc.y >= overlay_size.y + y_position) {\n"
"        float4 val = read_imagef(main, sampler, loc);\n"
"        write_imagef(dst, loc, val);\n"
"    } else {\n"
"        int2 loc_overlay    = (int2)(x_position, y_position);\n"
"        float4 in_main      = read_imagef(main,    sampler, loc);\n"
"        float4 in_overlay_u = read_imagef(overlay_u, sampler, loc - loc_overlay);\n"
"        float4 in_overlay_v = read_imagef(overlay_v, sampler, loc - loc_overlay);\n"
"        float4 in_overlay   = (float4)(in_overlay_u.x, in_overlay_v.x, 0.0f, 1.0f);\n"
"\n"
"        int2 loc_alpha      = (int2)(loc.x * alpha_adj_x, loc.y * alpha_adj_y) - loc_overlay;\n"
"        float4 in_alpha     = read_imagef(alpha,   sampler, loc_alpha);\n"
"\n"
"#ifdef NEED_UNPREMUL\n"
"        in_overlay.xy = (in_alpha.x > 0.0f && in_alpha.x < 1.0f)\n"
"                        ? min((in_overlay.xy - 0.5f) * 1.0f / in_alpha.x + 0.5f, 1.0f)\n"
"                        : in_overlay.xy;\n"
"#endif\n"
"        float4 val = in_overlay * in_alpha.x + in_main * (1.0f - in_alpha.x);\n"
"        write_imagef(dst, loc, val);\n"
"    }\n"
"}\n"
;
